local ADDON, data = ...

SanityCheck_data = data

local RELEVANT_LEVEL = 90
local GEAR = 'Gear'
local AURA = 'Aura'

local tblOops = {}              -- Primary warnings: bad stats, missing auras
local tblOops2 = {}             -- Secondary warnings: missing gems, missing enchants
local tblOops3 = {}             -- Tertiary warnings: missing buffs
local tblOops4 = {}             -- <What word comes next...?> warnings: consumables
local tblShoppingListGems = {}
local tblShoppingListEnch = {}
local tooltip = ''

local PLAIN_LETTER = 8383   -- For use in generating simple tooltips
local LibWeagleTooltip = LibStub("LibWeagleTooltip-2.1")

local THROTTLE = 1
local inCombat = false
local class

local _G = _G

------------------------------------------------------------------
--      iter_cycle
do
    local function next (self)
        local ret = self.values[self.cur]
        self.cur = self.cur + 1
        if self.cur>#self.values then
            self.cur = 1
        end
        return ret
    end
   
    function iter_cycle (...)
        local obj = {
            values = {...},
            cur = 1,
            next = next,
        }
      
        return obj
    end   
end

------------------------------------------------------------------

local tblProfessions = {}
local function InitProfessions ()
    local prof1, prof2 = GetProfessions()

    for i,v in ipairs ({prof1,prof2}) do
        local name, _, rank, maxRank, numSpells, _, skillLine = GetProfessionInfo(v)
        if rank>=600 then
            tblProfessions[skillLine] = true
        end
    end
end
------------------------------------------------------------------

local currentRaidDifficulty = 0
local function UpdateRaidDifficulty ()
    local _,_, difficultyID,_,_,_,_, instanceMapID = GetInstanceInfo()
    currentRaidDifficulty = data.difficultyFromID[difficultyID]
end

------------------------------------------------------------------

local tblWrongStats = {}
local myRole = ''
-- These will be set to point at the correct sets of wrong stats
local function UpdateWrongStats ()
    local spec = _G.GetSpecialization()

    if data.classification[class]==nil  or  spec==nil then 
        tblWrongStats = {}
        myRole = ''
    else
        tblWrongStats = data.wrongStats[data.classification[class][spec]] or {}
        myRole = data.classification[class][spec]
    end
end
------------------------------------------------------------------

local function MyGetItemStats (itemLink)
    if not itemLink then return end
    
    local tt = CreateFrame ("GameTooltip", 'temptooltip', UIParent, "GameTooltipTemplate")
    tt:SetOwner(UIParent, "ANCHOR_PRESERVE")
    tt:SetPoint("CENTER", "UIParent")
    tt:Hide()

    tt:SetOwner (UIParent, "ANCHOR_PRESERVE")
    tt:SetHyperlink ("spell:1:temptooltip")
    tt:Show ()
    tt:SetHyperlink (itemLink)

    local tmp = {}
    for i=1, tt:NumLines() do
        table.insert (tmp, _G['temptooltipTextLeft'..i]:GetText ())
    end
    
    local val, stat
    local ret = {}
    for i=1, tt:NumLines() do
        val,stat = tmp[i]:match ('^\+(%d+) ([a-zA-Z ]+)') 
        if val then 
            ret[stat] = val
        end
    end
    tt:Hide ()
    
    return ret
end

------------------------------------------------------------------

local function VerifyItemStats (itemLink)
    -- Check against the bad stats table, return false for a bad item.
    local tblStats = _G.GetItemStats (itemLink)  or  {}     -- The {} is just to prevent a stupid login error, really
    
    -- Override the armor bit because I only want bonus armor and Blizzard decided to go full retard
    local myStats = MyGetItemStats (itemLink)
    tblStats['RESISTANCE0_NAME'] = myStats['Bonus Armor']
    
    -- Look for outright wrong stats
    local count = 0
    for stat,statVal in _G.pairs(tblStats) do
        count = count + 1
        -- Check wrong stats
        for i,wrongStat in _G.pairs (tblWrongStats) do
            if stat==wrongStat then
                return false
            end
        end
    end
    
    -- Check if this item is meant strictly for another spec, because it has their primary stat but not yours
    if myStats[data.statNames[data.primaryStats[myRole]]]==nil then
        for i,v in ipairs (data.primaryWrongStats[myRole]) do
            if myStats[data.statNames[v]]~=nil then
                return false
            end
        end
    end
    
    -- Super special case: an item that has only stamina, for a non-tank
    if count==1 and myStats['Stamina'] and myRole~='TANK_STR' and myRole~='TANK_AGI' then
        return false
    end
    
    return true
end

------------------------------------------------------------------

local function VerifyItemType (itemLink)
    -- For high level toons, also check ilvl and quality. The ilvl threshold is pretty arbitrary.
    -- Return false for a bad item.
    if _G.UnitLevel ('player') >= RELEVANT_LEVEL then
        local _,_,qual, ilvl = _G.GetItemInfo (itemLink)
        if qual==nil or ilvl==nil then      -- This seems to happen occasionally... must be a bug of some sort of while logging on...
            return true
        end
        if qual<=1 or ilvl<=300 then
            return false
        end
    end
    
    return true
end

------------------------------------------------------------------

local function ShoppingList_CountSockets (itemLink)
    if not itemLink then return end
    
    local _,_,_, ilvl,_,_,_,_,equipSlot = _G.GetItemInfo (itemLink)
    if ilvl==nil or ilvl < g_SanityCheck_config['GemMinLvl'] then return end

    
    local tmp
    local i = 1
    while true do
        tmp = LibWeagleTooltip:GetTooltipLine(itemLink, i)
        if not tmp then 
            break
        else 
            if tmp:match ('Socket') and not tmp:match('Bonus') then 
                tblShoppingListGems[tmp] = (tblShoppingListGems[tmp] or 0) + 1
            end            
            i = i+1
        end
    end
end

------------------------------------------------------------------

local function CountSockets_Total (itemLink)
    if itemLink==nil then return 0 end
    local tblStats = _G.GetItemStats (itemLink)
    local ret = 0

    if tblStats==nil then tblStats={} end -- Just to prevent a stupid login error, really
    for k,v in _G.pairs(tblStats) do
        if k:match('SOCKET') then 
            ret = ret+v
        end
    end
    
    return ret
end

------------------------------------------------------------------

local function CountSockets_Gemmed (itemLink)
    if itemLink==nil then return 0 end
    local itemId, enchantId, gem1, gem2, gem3, gem4 = itemLink:match("item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)")
    
    local ret = 0
    for i,v in _G.ipairs({gem1,gem2,gem3,gem4}) do
        if v~='0' then
            ret = ret+1
        end
    end
    
    return ret
end

------------------------------------------------------------------

local function StringHasBadStat (str)
    if str==nil then 
        return false
    end

    for i,stat in _G.pairs(tblWrongStats) do
        if str:match (data.statNames[stat]) and not (str=='Enchanted: Jade Spirit' and stat=='ITEM_MOD_SPIRIT_SHORT') then    -- Sorry for this horrible lame workaround but OH GOD IT'S SO ANNOYING ><
            return true
        end
    end
    
    return false
end

------------------------------------------------------------------

local function VerifySocketAndEnchantStats (itemLink)
    -- Generate a tooltip of a plain letter (statless, cached item) with the same enchant and gems as this item
    local itemId, enchantId, gem1, gem2, gem3, gem4 = itemLink:match("item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+)")
    local tooltip = ("item:%i:%i:%i:%i:%i:%i"):format(PLAIN_LETTER, enchantId, gem1, gem2, gem3, gem4)

    -- Get the enchant from line 2 of the tooltip, and up to 4 gems from the next 4 lines
    local tmp = {}  -- This table will contain the enchant text, followed by up to 4 gem texts
    for i=2,6 do
        tmp[i-1] =  LibWeagleTooltip:GetTooltipLine(tooltip, i)
    end
    
    for k,v in _G.pairs(tmp) do
        if StringHasBadStat(v) then
            return false
        end
    end

    return true
end

------------------------------------------------------------------

local function VerifyItemSockets (itemLink)
    local _,_,_, ilvl,_,_,_,_,equipSlot = _G.GetItemInfo(itemLink)
    
    if ilvl==nil or ilvl < g_SanityCheck_config['GemMinLvl'] then
        return true
    else
        return (CountSockets_Total(itemLink) <= CountSockets_Gemmed(itemLink))
    end
end

------------------------------------------------------------------

local function VerifyItemEnchant (itemLink)
    local _,_,_, ilvl = _G.GetItemInfo (itemLink)
    
    if ilvl==nil or ilvl < g_SanityCheck_config['EnchMinLvl'] then
        return true
    else
        local itemId, enchantId = itemLink:match ("item:(%d+):(%d+)")
        return enchantId~='0'
    end
end

------------------------------------------------------------------

local function VerifyRoleAndSpec ()
    local spec = _G.GetSpecialization()
    local role = _G.UnitGroupRolesAssigned ('player')
    
    if data.roles[class]==nil or spec==nil or _G.UnitGroupRolesAssigned ('player')=='NONE' then return true,'' end
    
    if data.roles[class][spec] == role then
        return true, ''
    else
        local _, spec = _G.GetSpecializationInfo (_G.GetSpecialization())
        return false, 'Are you sure you want to '..data.verbs[role]..' in '..spec..' spec?'
    end
end

------------------------------------------------------------------

local function VerifyProfessions_Engineering ()
    return _G.GetItemSpell (_G.GetInventoryItemLink ('player',INVSLOT_WAIST)) and _G.GetItemSpell (_G.GetInventoryItemLink ('player',INVSLOT_BACK))
end

------------------------------------------------------------------

local function VerifyFlask ()
    if (not g_SanityCheck_config.ReadyToRaid.Flask[currentRaidDifficulty]) then
        return true,''
    end
    
    for _,name in _G.ipairs(data.flasks) do
        if HasAura ('player', name) then
            return true,''
        end
    end
    return false, 'Don\'t forget to eat a flask!'
end

------------------------------------------------------------------

local function VerifyFood ()
    -- Verify that either the player has the Well Fed buff, or does not wish to be reminded of it in the current difficulty
    if (not g_SanityCheck_config.ReadyToRaid.Food[currentRaidDifficulty]) or HasAura ('player', 'Well Fed') then
        return true,''
    end
    return false, 'You look hungry!'
end

------------------------------------------------------------------

local function CallVerifyInv (func, link, tbl, msg)            -- A wrapper for inventory verification functions
    if not func(link) then
        _G.table.insert (tbl, msg)
    end
end
------------------------------------------------------------------

local function CheckInventory ()
    for slot,slotName in _G.pairs(data.slots) do
        local link = _G.GetInventoryItemLink ('player', slot)
        if link~=nil then
            CallVerifyInv (VerifyItemType, link,                tblOops[GEAR],      'Low level item: '..slotName)
            CallVerifyInv (VerifyItemSockets, link,             tblOops2[GEAR],     'Missing gems: '..slotName)
            CallVerifyInv (VerifySocketAndEnchantStats, link,   tblOops2[GEAR],     'Bad gems/ enchant: '..slotName)
            
            if not g_SanityCheck_config.DontCheckStats then
                CallVerifyInv (VerifyItemStats, link,           tblOops[GEAR],      'Bad item: '..slotName)
            end
            
            if data.slots_Enchantable[slot] then
                CallVerifyInv (VerifyItemEnchant, link,             tblOops2[GEAR],     'Missing enchant: '..slotName)
            end
        else
            -- Mention if the item is missing - unless it's an offhand, and there's a 2H equipped
            -- Low level toons can get away with empty slots
            if (slot==INVSLOT_OFFHAND and Has2HanderEquipped()) or (_G.UnitLevel('player')<60) then
                -- All is well
            else
                _G.table.insert (tblOops[GEAR], 'Missing item: '..slotName)
            end
        end
    end
    
    if tblProfessions[data.professionIds.Engineering] then
        CallVerifyInv (VerifyProfessions_Engineering, nil, tblOops2[GEAR], 'Don\'t forget to put some engineering gizmos on your gear!')
    end
end

------------------------------------------------------------------

local function CallVerifyAura (func, tbl)      -- A wrapper for aura verification functions
    local ret, msg = func()
    if not ret then
        _G.table.insert (tbl, msg)
    end
end

------------------------------------------------------------------

local function CheckAuras ()
    local spec = _G.GetSpecialization()
    
    -- Personal auras and stuff
    if data.verifyAuras[class]==nil or spec==nil then return end
    
    local funcs
    local tmp = data.verifyAuras[class][spec]
    if _G.type(tmp)=='function' then
        funcs = {tmp}
    else
        funcs = tmp
    end
    
    for i,func in _G.ipairs(funcs) do
        CallVerifyAura (func, tblOops[AURA])
    end
    
    -- Raid buffs
    if data.verifyRaidBuffs[class] then
        CallVerifyAura (data.verifyRaidBuffs[class], tblOops3[AURA])
    end
 
    -- Role
    CallVerifyAura (VerifyRoleAndSpec,  tblOops[AURA])
    
    -- Ready to raid
    CallVerifyAura (VerifyFlask,        tblOops4[AURA])
    CallVerifyAura (VerifyFood,         tblOops4[AURA])

end

------------------------------------------------------------------

local function Init ()
    -- Main frame thing. This controls all the lights but is not visible in itself, to allow for skins etc.
    do
        local f = CreateFrame ("Frame", 'frmSanityCheck', UIParent)
        
        f.needGearScan = false
        f.needAuraScan = false
        
        f.elapsed2 = 0
        
        function f:GetTooltipText_Light1 ()
            local ret = {}
            for key,subtbl in _G.pairs(tblOops) do
                if #subtbl>0 then
                    for k,v in pairs(subtbl) do
                        table.insert (ret, v)
                    end
                end
            end
            return ret
        end


        function f:GetTooltipText_Light2 ()
            local ret = {}
            for key,subtbl in _G.pairs(tblOops2) do
                for k,v in pairs(subtbl) do
                    table.insert (ret, v)
                end
            end
            return ret
        end


        function f:GetTooltipText_Light3 ()
            local ret = {}
            
            for key,subtbl in _G.pairs(tblOops3) do      -- Iterating over the GEAR and AURA sub-tables
                for k,v in pairs(subtbl) do
                        table.insert (ret, v)
                end
            end
            return ret
        end


        function f:GetTooltipText_Light4 ()
            local ret = {}
            for key,subtbl in _G.pairs(tblOops4) do      -- Iterating over the consumables table
                for k,v in pairs(subtbl) do
                    table.insert (ret, v)
                end
            end
            return ret
        end
        
    end
end

------------------------------------------------------------------
--  Events and such
local function RegisterEvents ()
    local f = frmSanityCheck
    function f:Scan ()
        -- Aura scanning is much lighter than gear scanning, so we always do it
        
        -- Clear tables
        tblOops[AURA] = {}
        tblOops2[AURA] = {}
        tblOops3[AURA] = {}
        tblOops4[AURA] = {}
        if self.needGearScan then
            tblOops[GEAR] = {}
            tblOops2[GEAR] = {}
            tblOops3[GEAR] = {}
            tblOops4[GEAR] = {}
            tblShoppingListGems = {}
            tblShoppingListEnch = {}
        end

        -- Now do stuff
        if self.needGearScan and (not inCombat) then
            CheckInventory ()
            self.needGearScan = false
        end
            
        CheckAuras ()
        self.needAuraScan = false
        
        if #tblOops[GEAR] > 0 or #tblOops[AURA] > 0 then
            frmSanityCheck_Light1:Light ()
        else
            frmSanityCheck_Light1:Dark ()
        end

        if #tblOops2[GEAR] > 0 or #tblOops2[AURA] > 0 then
            frmSanityCheck_Light2:Light ()
        else
            frmSanityCheck_Light2:Dark ()
        end

        if #tblOops3[GEAR] > 0 or #tblOops3[AURA] > 0 then
            frmSanityCheck_Light3:Light ()
        else
            frmSanityCheck_Light3:Dark ()
        end        
        
        if #tblOops4[GEAR] > 0 or #tblOops4[AURA] > 0 then
            frmSanityCheck_Light4:Light ()
        else
            frmSanityCheck_Light4:Dark ()
        end
        
        if g_SanityCheck_config.HideWhenAwesome and (not frmSanityCheck_Light1.isOn) and (not frmSanityCheck_Light2.isOn) and (not frmSanityCheck_Light3.isOn) and (not frmSanityCheck_Light4.isOn) then
            frmSanityCheck_Parent:Hide ()
        else
            frmSanityCheck_Parent:Show ()
        end
    end
    
    f.needGearScan = true
    f.needAuraScan = true
    UpdateRaidDifficulty ()
    inCombat = _G.UnitAffectingCombat('player')
    
    --f:RegisterUnitEvent ("UNIT_AURA", 'player')
    f:RegisterEvent ("UNIT_AURA")
    f:RegisterUnitEvent ("UNIT_PET", 'player')
    f:RegisterEvent ("UNIT_INVENTORY_CHANGED")
    f:RegisterEvent ("SOCKET_INFO_UPDATE")
    f:RegisterEvent ("PLAYER_SPECIALIZATION_CHANGED")
    f:RegisterEvent ("PET_BAR_UPDATE")
    f:RegisterEvent ("GROUP_ROSTER_UPDATE")   -- To detect role change!
    f:RegisterEvent ("PLAYER_REGEN_ENABLED")
    f:RegisterEvent ("PLAYER_REGEN_DISABLED")
    f:RegisterEvent ("ZONE_CHANGED")
    f:RegisterEvent ("ZONE_CHANGED_NEW_AREA")
    f:RegisterEvent ("PLAYER_DIFFICULTY_CHANGED")
    
    f:RegisterEvent ("START_AUTOREPEAT_SPELL")
    f:RegisterEvent ("STOP_AUTOREPEAT_SPELL")
    f:RegisterEvent ("PET_BAR_UPDATE")
    
    
    f:SetScript ("OnEvent", function  (self, event, GUID, ...)
        if event=='PLAYER_REGEN_DISABLED' then
            inCombat = true
        elseif event=='PLAYER_REGEN_ENABLED' then
            inCombat = false
        elseif event=='UNIT_INVENTORY_CHANGED' then
            self.needGearScan = true
        elseif event=='UNIT_PET' or event=='UNIT_AURA' or event=='PET_BAR_UPDATE' then
            self.needAuraScan = true
        elseif event=='PLAYER_SPECIALIZATION_CHANGED' then
            UpdateWrongStats ()
            self.needGearScan = true
            self.needAuraScan = true
        elseif event=='ZONE_CHANGED' or event=='ZONE_CHANGED_NEW_AREA' or event=='PLAYER_DIFFICULTY_CHANGED' then
            UpdateRaidDifficulty ()
        end
    end)
    function f:OnUpdate (elapsed, ...)
        self.elapsed2 = self.elapsed2 + elapsed
        
        --if self.elapsed2 > THROTTLE and (self.needGearScan or self.needAuraScan) then
        if ((not self.needGearScan) and self.needAuraScan and self.elapsed2 > THROTTLE)  or  (self.needGearScan and self.elapsed2 > THROTTLE * 3) then
            self:Scan ()
            self.elapsed2 = 0
            if (not self.isOn) and (not self.frmLight2.isOn) and (not self.frmLight3.isOn) and (not self.frmLight4.isOn) and g_SanityCheck_config.HideWhenAwesome then
                self:Hide ()
            else
                self:Show ()
            end
        end
    end

    local updater = CreateFrame ("Frame", '', UIParent)
    updater:SetScript ("OnUpdate", function (self, elapsed, ...)
        frmSanityCheck:OnUpdate (elapsed)
    end)
end


------------------------------------------------------------------

function CreateShoppingList ()
    -- A point!
    -- Because the frame gets anchored to the fontstring, and fontstrings apparently can't be moved (?)
    local point = CreateFrame ("Frame", 'frmShoppingList_point', UIParent)
    point:SetSize (1,1)
    point:SetPoint ("CENTER", UIParent, "CENTER")
    
    local f = CreateFrame ("Frame", 'frmShoppingList', UIParent)
    f:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        edgeSize = 8,
    })

    f.lbl = f:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
    f.lbl:SetPoint ("CENTER", point, "CENTER")
    f.lbl:SetTextColor (1.0,1.0,1.0)
    f.lbl:SetJustifyH ("LEFT")
    
    f:SetPoint ("TOPLEFT", f.lbl, "TOPLEFT", -12, 12)
    f:SetPoint ("BOTTOMRIGHT", f.lbl, "BOTTOMRIGHT", 12, -12)
    f:Hide ()
    
    -- Make movable
    point:SetMovable (true)
    point:RegisterForDrag ("LeftButton")
    point:SetUserPlaced (true)
    point:SetClampedToScreen (true)
    point:EnableMouse (true)
    f:SetScript("OnMouseDown", function(self, button)
        if button=='LeftButton' then
            frmShoppingList_point:StartMoving ()
        elseif button=='RightButton' then
            frmShoppingList_point:StopMovingOrSizing ()
            self:Hide ()
        end
    end)
    f:SetScript("OnMouseUp", function(self, button)
        frmShoppingList_point:StopMovingOrSizing ()
    end)

    function f:Generate ()
        tblShoppingListGems = {}
        tblShoppingListEnch = {}

        for slot,slotName in _G.pairs(data.slots) do
            local link = _G.GetInventoryItemLink ('player', slot)
            if link~=nil then
                if data.slots_Enchantable[slot] and (not VerifyItemEnchant(link)) then
                    tblShoppingListEnch[slotName] = 1
                end
                
                ShoppingList_CountSockets (link)
            end
        end    
    
        local txt = 'Empty sockets:\n'
        for k,v in pairs(tblShoppingListGems) do
            txt = txt..v..'x  '..k..'\n'
        end
        txt = txt..'\nMissing enchants:\n'
        for k,v in pairs(tblShoppingListEnch) do
            txt = txt..k..'\n'
        end
        self.lbl:SetText (txt)
        
        self:Show ()
    end
end

------------------------------------------------------------------
------------------------------------------------------------------

g_SanityCheck_config = g_SanityCheck_config or {}

Init ()

local starter = CreateFrame ("Frame")
starter:RegisterEvent ("PLAYER_LOGIN")
starter:RegisterEvent ("ADDON_LOADED")
starter:SetScript ("OnEvent", function  (self, event, ...)
    if event=="PLAYER_LOGIN" then
        RegisterEvents ()
        InitProfessions ()
        frmSanityCheck.needGearScan = true
        frmSanityCheck.needAuraScan = true
        UpdateRaidDifficulty ()
        inCombat = _G.UnitAffectingCombat('player')
        class = _G.UnitClass('player')
        UpdateWrongStats ()
    elseif event=="ADDON_LOADED" then
        local arg1 = ...
        if arg1==ADDON then
            --BuildGui ()
            data.skins[g_SanityCheck_config.skin or 'Classic'].build ()
            CreateShoppingList ()
            SanityCheck_CreateOptions ()
            UpdateWrongStats () -- Yes, I need to call it on both events because who knows which one happens first, and I need both, and ugh.
        end
    end
end)

------------------------------------------------------------------
------------------------------------------------------------------
-- Skin Dev

do
    local f = CreateFrame ("Frame", 'frmSkinDev', UIParent)
    f:SetSize (360,160)
    f:SetPoint ("LEFT", UIParent, "LEFT", 100, 0)
    f:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        edgeSize = 8,
    })
    
    function f:Apply ()
        frmSanityCheck_Parent:SetSize (tonumber (f.Parent.w:GetText ()), tonumber (f.Parent.h:GetText ()))
        frmSanityCheck_Light1:SetSize (tonumber (f.Light1.w:GetText ()), tonumber (f.Light1.h:GetText ()))
        frmSanityCheck_Light2:SetSize (tonumber (f.Light2.w:GetText ()), tonumber (f.Light2.h:GetText ()))
        frmSanityCheck_Light3:SetSize (tonumber (f.Light3.w:GetText ()), tonumber (f.Light3.h:GetText ()))
        frmSanityCheck_Light4:SetSize (tonumber (f.Light4.w:GetText ()), tonumber (f.Light4.h:GetText ()))
        frmSanityCheck_Light1:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", tonumber (f.Light1.x:GetText ()), tonumber (f.Light1.y:GetText ()))
        frmSanityCheck_Light2:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", tonumber (f.Light2.x:GetText ()), tonumber (f.Light2.y:GetText ()))
        frmSanityCheck_Light3:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", tonumber (f.Light3.x:GetText ()), tonumber (f.Light3.y:GetText ()))
        frmSanityCheck_Light4:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", tonumber (f.Light4.x:GetText ()), tonumber (f.Light4.y:GetText ()))
    end
    
    function f:CreateLabel (text, x, y)
        local ret = self:CreateFontString (nil, "BACKGROUND", "GameFontNormal")
        ret:SetPoint ("TOPLEFT", self, "TOPLEFT", x, y)
        ret:SetTextColor (1.0,1.0,1.0)
        ret:SetJustifyH ("LEFT")
        ret:SetText (text)
        
        return ret
    end
    
    function f:CreateTextbox (x, y)
        local ret = CreateFrame ("EditBox", nil, self)
        ret:SetBackdrop ({
            bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
            edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
            tile = true,
            tileSize = 32,
            edgeSize = 8,
        })
        ret:SetPoint ("TOPLEFT", self, "TOPLEFT", x, y)
        ret:SetSize (40,20)
        ret:SetTextColor (1,1,1)
        ret:SetFont ('Fonts\\ARIALN.TTF', 11, "")
        ret:SetAutoFocus (false)
        ret:SetTextInsets (4, 4, 0, 0)
        ret:SetScript ("OnEscapePressed", ret.ClearFocus)
        --ret:SetNumeric (true)
        
        ret:SetScript ("OnEnterPressed", f.Apply)
        ret:SetScript ("OnTabPressed", function (self)
            if IsLeftShiftKeyDown() or IsRightShiftKeyDown() then
                if self.prev then self.prev:SetFocus () end
            else
                if self.next then self.next:SetFocus () end
            end
        end)
        ret:SetScript ("OnEditFocusGained", function (self)
            self:HighlightText ()
        end)
        ret:SetScript ("OnEditFocusLost", function (self)
            self:HighlightText (0,0)
        end)
        
        if self.lastTextbox then
            ret.prev = self.lastTextbox
            ret.prev.next = ret
        end
        
        self.lastTextbox = ret
        return ret
    end
    
    function f:CreateButton (x, y, idx, func)
        local ret = CreateFrame ("Button", nil, self, "UIPanelButtonTemplate")
        ret:SetSize (20,20)
        ret:SetPoint ("TOPLEFT", self, "TOPLEFT", x, y)
        ret.idx = idx
        ret:SetScript ("OnClick", func)
        
        return ret
    end
    
    -- Create all the textboxes and buttons and stuff
    local t = {'Parent', 'Light1', 'Light2', 'Light3', 'Light4'}
    for i,v in ipairs(t) do
        local y = 15-30*i
        f[v] = {}
        f:CreateLabel (v, 10, y)
        f:CreateLabel ('Size:', 60, y)
        f[v].w = f:CreateTextbox (95, y+3)
        f[v].h = f:CreateTextbox (145, y+3)
        if i>1 then
            f:CreateLabel ('Pos:', 195, y)
            f[v].x = f:CreateTextbox (230, y+3)
            f[v].y = f:CreateTextbox (280, y+3)
            
            f[v].btn = f:CreateButton (330, y+3, i-1, function (self,...)
                local b = _G['frmSanityCheck_Light'..self.idx]
                if b.isOn then
                    _G['frmSanityCheck_Light'..self.idx]:Dark ()
                else
                    _G['frmSanityCheck_Light'..self.idx]:Light ()
                end
            end)
        end
    end
    
    f:SetMovable (true)
    f:RegisterForDrag ("LeftButton")
    f:SetUserPlaced (true)
    f:SetClampedToScreen (true)
    f:EnableMouse (true)
    f:SetScript("OnMouseDown", function(self, button)
        if button=='LeftButton' then
            self:StartMoving ()
        end
    end)
    f:SetScript("OnMouseUp", function(self, button)
        self:StopMovingOrSizing ()
    end)
    
    function f:LoadData (data)
        local t = {Parent='bg', Light1='light1', Light2='light2', Light3='light3', Light4='light4'}
        for k,v in pairs(t) do
            f[k].w:SetText (data[v].w)
            f[k].h:SetText (data[v].h)
            if k~='Parent' then
                f[k].x:SetText (data[v].x)
                f[k].y:SetText (data[v].y)
            end
        end
    end

    f:Hide ()
end

SLASH_SKIN1 = "/skin"
SlashCmdList["SKIN"] = function (txt)
    if frmSkinDev:IsShown () then
        frmSkinDev:Hide ()
    else
        frmSkinDev:Show ()
    end
end


function SanityCheck_SimpleSkin (data)
    -- The data parameter contains a bunch of things. Please refer to an example.
    local SkinName = data.name

    local function CreateLight (name, parent, texture)
        local f = CreateFrame ("Frame", name, parent)
        f:SetBackdrop ({bgFile = "Interface\\Addons\\"..ADDON.."\\gfx\\"..texture})
        
        f:SetScript("OnEnter", function(self, motion, ...)
            GameTooltip:SetOwner (self, "ANCHOR_TOP")
            self:GenerateTooltip ()
            GameTooltip:Show ()
        end)
        f:SetScript("OnLeave", function(self, motion, ...)
            GameTooltip:Hide ()
        end)
        
        function f:Light ()
            self:SetAlpha (1)
            self.isOn = true
        end
        function f:Dark ()
            self:SetAlpha (0)
            self.isOn = false
        end
        f:Dark()
        
        return f
    end

    local function BuildGui (x,y)
        -- Main frame
        do  
            local d = data.bg
            local f = CreateFrame ("Frame", 'frmSanityCheck_Parent', UIParent)
            f:SetSize (d.w, d.h)
            f:SetPoint ("CENTER", UIParent, "BOTTOMLEFT", x or 100, y or 100)
            f:SetBackdrop ({bgFile = "Interface\\Addons\\"..ADDON.."\\gfx\\".. d.img})
            
            -- Since this one is the main frame, we need the whole movability thing
            f:SetMovable (true)
            f:RegisterForDrag ("LeftButton")
            f:SetUserPlaced (true)
            --f:SetClampedToScreen (true)
            f:EnableMouse (true)
            f:SetScript("OnMouseDown", function(self, button)
                if button=='LeftButton' then
                    self:StartMoving ()
                    GameTooltip:Hide ()
                elseif button=='RightButton' then
                    
                end
            end)
            f:SetScript("OnMouseUp", function(self, button)
                self:StopMovingOrSizing ()
            end)

            frmSanityCheck_Parent = f
        end
        
        -- Primary warning light
        do
            local d = data.light1
            local f = CreateLight ('frmSanityCheck_Light1', frmSanityCheck_Parent, d.img)
            f:SetSize (d.w, d.h)
            f:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", d.x, d.y)
            
            function f:GenerateTooltip ()
                if self.isOn then
                    GameTooltip:AddDoubleLine ('SanityCheck', '', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                    
                    local txt = frmSanityCheck.GetTooltipText_Light1 ()
                    for k,v in pairs(txt) do
                        GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                    end
                else
                    GameTooltip:AddLine ('SanityCheck', 1.0, 1.0, 1.0)
                    GameTooltip:AddLine ('Your gear and buffs appear to be fine!', 1.0, 0.5, 0.5)
                end
            end
            
            frmSanityCheck.frmLight1 = f
            frmSanityCheck_Light1 = f
        end
        
        -- Secondary warning light
        do
            local d = data.light2
            local f = CreateLight ('frmSanityCheck_Light2', frmSanityCheck_Parent, d.img)
            f:SetSize (d.w, d.h)
            f:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", d.x, d.y)
            
            function f:GenerateTooltip ()
                if self.isOn then
                    GameTooltip:AddDoubleLine ('SanityCheck', 'Right-click to generate shopping list', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                    
                    local txt = frmSanityCheck.GetTooltipText_Light2 ()
                    for k,v in pairs(txt) do
                        GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                    end
                else
                    GameTooltip:AddLine ('SanityCheck', 1.0, 1.0, 1.0)
                    GameTooltip:AddLine ('Your gear and buffs appear to be fine!', 1.0, 0.5, 0.5)
                end
            end
            
            f:SetScript("OnMouseDown", function(self, button)
                if button=='RightButton' then
                    frmShoppingList:Generate()
                end
            end)
           
            frmSanityCheck.frmLight2 = f
            frmSanityCheck_Light2 = f
        end

        -- Tertiary warning light
        do
            local d = data.light3
            local f = CreateLight ('frmSanityCheck_Light3', frmSanityCheck_Parent, d.img)
            f:SetSize (d.w, d.h)
            f:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", d.x, d.y)
            
            function f:GenerateTooltip ()
                if self.isOn then
                    GameTooltip:AddDoubleLine ('SanityCheck', 'Even More Warnings', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                    
                    local txt = frmSanityCheck.GetTooltipText_Light3 ()
                    for k,v in pairs(txt) do
                        GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                    end
                else
                    GameTooltip:AddLine ('SanityCheck', 1.0, 1.0, 1.0)
                    GameTooltip:AddLine ('Nothing to see here, move along now!', 1.0, 0.5, 0.5)
                end
            end
            
            frmSanityCheck.frmLight3 = f
            frmSanityCheck_Light3 = f
        end

        -- 4th warning light
        do
            local d = data.light4
            local f = CreateLight ('frmSanityCheck_Light4', frmSanityCheck_Parent, d.img)
            f:SetSize (d.w, d.h)
            f:SetPoint ("TOP", frmSanityCheck_Parent, "TOP", d.x, d.y)
            
            function f:GenerateTooltip ()
                if self.isOn then
                    GameTooltip:AddDoubleLine ('SanityCheck', 'Consumables', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                
                    local txt = frmSanityCheck.GetTooltipText_Light4 ()
                    for k,v in pairs(txt) do
                        GameTooltip:AddLine (v, 1.0, 0.5, 0.5)
                    end
                else
                    GameTooltip:AddDoubleLine ('SanityCheck', 'Missing consumables', 1.0, 1.0, 1.0, 1.0, 1.0, 0.0)
                    GameTooltip:AddLine ('Looks like you\'ve got everything you need!', 1.0, 0.5, 0.5)
                end
            end
            
            frmSanityCheck.frmLight4 = f
            frmSanityCheck_Light4 = f
        end
        
        frmSkinDev:LoadData (data)
    end

    SanityCheck_data.skins = SanityCheck_data.skins or {}
    SanityCheck_data.skins[SkinName] = {
        ['build'] = BuildGui
    }
end